/*
    Copyright 2008,2009 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>
#include <SDL.h>        // http://www.libsdl.org
#include <SDL_image.h>  // http://www.libsdl.org/projects/SDL_image/
#include <SDL_rwops.h>
#include "mydownlib.h"
#include "resize.h"     // http://members.cox.net/dolsen6/resize.zip

#ifdef WIN32
    #include <direct.h>
    #include <windows.h>
    #define make_dir(x) mkdir(x)
    #define PATH_SLASH  '\\'
#else
    #define stricmp     strcasecmp
    #define strnicmp    strncasecmp
    #define make_dir(x) mkdir(x, 0755)
    #define PATH_SLASH  '/'
#endif

typedef Uint8   u8;
typedef Uint32  u32;



#define VER         "0.1.2b"
#define SHOWIT      69
#define MAXFMT      1024
#define RESIZEQUAL  3
#define PATHMAX     4096    // MAX_PATH + MAX_FNAME

#define FULLSCREEN_JOB \
                    xcur = x; \
                    ycur = y; \
                    x = y = 0; \
                    if(modes) { \
                        for(i = 0; modes[i]; i++) { \
                            if((wold > modes[i]->w) || (hold > modes[i]->h)) break; \
                            x = modes[i]->w; \
                            y = modes[i]->h; \
                        } \
                        if(!x || !y) { \
                            x = modes[0]->w; \
                            y = modes[0]->h; \
                        } \
                        y = (int)((float)x / (float)ratio); \
                        if(!modes[i]) i--; \
                        if(y > modes[i]->h) { \
                            if(i >= 0) { \
                                x = modes[i]->w; \
                                y = modes[i]->h; \
                                x = (int)((float)y * (float)ratio); \
                            } \
                        } \
                    } else { \
                        x = wold; \
                        y = hold; \
                        y = (int)((float)x / (float)ratio); \
                    }



void show_examples(void);
int recursive_dir(u8 *filedir);
u8 *get_date(void);
int letsdownload(void *parameter);
int letsfile(u8 *fname);
int letsload(void *parameter);
int letsslideshow(void *parameter);
void do_fmt(void);
void dumpimg(u8 *data, int datasz);
void pushrefresh(int code);
void pushevent(int type, int code);
void pushfullscreen(void);
void write_watch_ever(void);
u32 get_num(u8 *str);
u32 mycrc(u8 *data, int size);
void sdl_err(void);
void std_err(void);
int my_mem_seek(SDL_RWops *context, int offset, int whence);
int my_mem_read(SDL_RWops *context, void *ptr, int size, int maxnum);
int my_mem_write(SDL_RWops *context, const void *ptr, int size, int num);
int my_mem_close(SDL_RWops *context);



SDL_RWops   rw;
int     ms          = 200,
        start       = 0,
        startb      = 0,
        loop        = 1,
        run         = 1,
        record      = 0,
        ready       = 1,
        slideshow   = 0,
        www         = 0,
        doresize    = 0,
        doskip      = 0,
        urllen;
u8      *url,
        *fmt        = NULL;



int main(int argc, char *argv[]) {
    SDL_Surface *screen = NULL,
                *image  = NULL;
    SDL_Rect    **modes;
    SDL_Thread  *tid;
    SDL_Event   event;
    SDL_Rect    rec_on;
    float   ratio       = 1.0;
    u32     rec_color   = 0;
    int     i,
            fullscreen  = 0,
            x           = 0,    // current X/Y of the screen
            y           = 0,
            xcur        = 0,    // current X/Y, used for restoring the image from fullscreen
            ycur        = 0,
            wold        = 0,    // original X/Y of the image
            hold        = 0;
    u8      *p;

    setbuf(stdout, NULL);

    fputs("\n"
        "webimgms "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\n"
            "Usage: %s [options] <URL/file/folder>\n"
            "\n"
            "Options:\n"
            "-t MS   milliseconds to wait between each scan/refresh (%d)\n"
            "-f FMT  by default the program does a refresh of the same image/file/URL,\n"
            "        while with this option is possible to specify a sequence of images\n"
            "        based on the C printf style FMT formatter containing the name of the\n"
            "        file or its last part, use -e to see some examples\n"
            "-n NUM  related to the above option, needed to specify the first image number\n"
            "-s      slide-show option, will show any image in the given folder\n"
            "\n"
            "Other options:\n"
            "-F      start in fullscreen\n"
            "-l      related to -f, disable the automatic images loop when the images end\n"
            "-k      related to -f, if the image is not available continue the scanning\n"
            "        instead of restarting from the beginning\n"
            "-R      enable the resizing of the images (slow)\n"
            "-e      show some usage examples\n"
            "\n"
            "Note: when the tool starts will show the keys that you can press for enabling\n"
            "      the recording of the images (R), the full-screen (F) or quit (Q/ESC)\n"
            "\n", argv[0], ms);
        exit(1);
    }

    argc--;
    for(i = 1; i < argc; i++) {
        if(((argv[i][0] != '-') && (argv[i][0] != '/')) || (strlen(argv[i]) != 2)) {
            printf("\nError: wrong argument (%s)\n", argv[i]);
            exit(1);
        }
        switch(argv[i][1]) {
            case 't': {
                if(!argv[++i]) exit(1);
                ms          = atoi(argv[i]);
                } break;
            case 'f': {
                if(!argv[++i]) exit(1);
                fmt         = argv[i];
                } break;
            case 'n': {
                if(!argv[++i]) exit(1);
                start       = get_num(argv[i]);
                } break;
            case 's': {
                slideshow   = 1;
                } break;
            case 'l': {
                loop        = 0;
                } break;
            case 'k': {
                doskip      = 1;
                } break;
            case 'r': {
                doresize    = 0;
                } break;
            case 'R': {
                doresize    = SDL_RESIZABLE;
                } break;
            case 'e': {
                show_examples();
                } break;
            case 'F': {
                fullscreen  = SDL_FULLSCREEN;
                } break;
            default: {
                printf("\nError: wrong argument (%s)\n", argv[i]);
                exit(1);
            }
        }
    }

    urllen = strlen(argv[argc]) + MAXFMT;
    url = malloc(urllen);
    if(!url) std_err();
    strcpy(url, argv[argc]);
    if(!stricmp(url, "-e")) show_examples();

    printf("\n"
        "runtime keys:\n"
        " Q/ESC: quit\n"
        " R:     enable/disable the recording of the downloaded images\n"
        " F:     enable/disable fullscreen\n"
        "\n");

    putenv("SDL_VIDEO_WINDOW_POS=center");
    putenv("SDL_VIDEO_CENTERED=1");

    if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE)) sdl_err();
    atexit(SDL_Quit);
    SDL_WM_SetCaption("webimgms "VER, "webimgms "VER);

    memset(&rw, 0, sizeof(rw));
    rw.seek  = my_mem_seek;
    rw.read  = my_mem_read;
    rw.write = my_mem_write;
    rw.close = my_mem_close;
    startb   = start;

    modes = SDL_ListModes(NULL, SDL_FULLSCREEN | SDL_HWSURFACE);

    if(slideshow) {
        //for(p = url + strlen(url) - 1; p > url; p--) {
            //if((*p == '\\') || (*p == '/')) *p = 0;
        //}
        tid = SDL_CreateThread((void *)letsslideshow, NULL);
    } else if(!strnicmp(url, "http:", 5) || !strnicmp(url, "www.", 4)) {
        www = 1;
        tid = SDL_CreateThread((void *)letsdownload, NULL);
    } else {
        tid = SDL_CreateThread((void *)letsload, NULL);
    }

    if(fullscreen) {
        fullscreen = 0;
        pushfullscreen();
    }

    while(run) {
        if(!SDL_WaitEvent(&event)) continue;
        ready = 0;
        switch(event.type) {
            case SDL_VIDEORESIZE: {
                if(!doresize) break;
                x = event.resize.w;
                y = (int)((float)x / (float)ratio); //y = event.resize.h;
                SDL_FreeSurface(screen);
                screen = SDL_SetVideoMode(x, y, 0, SDL_ANYFORMAT | SDL_HWSURFACE | SDL_DOUBLEBUF | doresize | fullscreen);
                if(!screen) {
                    screen = SDL_SetVideoMode(x, y, 0, SDL_ANYFORMAT | SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_RESIZABLE | fullscreen);
                    if(!screen) sdl_err();
                }
                if(x != wold) image = SDL_ResizeXY(image, x, y, RESIZEQUAL);
                SDL_BlitSurface(image, NULL, screen, NULL);
                SDL_Flip(screen);
                break;
            }
            case SDL_USEREVENT: {
                if((event.user.code != SHOWIT) || !rw.hidden.mem.base) break;
                SDL_FreeSurface(image);
                image = IMG_Load_RW(&rw, 0);
                if(!image) {
                    fputc('x', stdout);
                    if(www) sdl_err();
                    break;
                }
                if(rw.hidden.mem.base) {
                    free(rw.hidden.mem.base);
                    rw.hidden.mem.base = NULL;
                }

                if((wold != image->w) || (hold != image->h)) {
                    wold = x = image->w;
                    hold = y = image->h;
                    SDL_FreeSurface(screen);
                    screen = SDL_SetVideoMode(x, y, 0, SDL_ANYFORMAT | SDL_HWSURFACE | SDL_DOUBLEBUF | doresize | fullscreen);
                    if(!screen) {
                        FULLSCREEN_JOB
                        screen = SDL_SetVideoMode(x, y, 0, SDL_ANYFORMAT | SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_RESIZABLE | fullscreen);
                        if(!screen) sdl_err();
                    }
                    ratio = (float)x / (float)y;
                    rec_color = SDL_MapRGB(image->format, 255, 0, 0);
                }

                if(record) {
                    rec_on.x = 4;
                    rec_on.y = 4;
                    rec_on.w = 8;
                    rec_on.h = 8;
                    SDL_FillRect(image, &rec_on, rec_color);
                }
                if(doresize && (x != wold)) image = SDL_ResizeXY(image, x, y, RESIZEQUAL);
                SDL_BlitSurface(image, NULL, screen, NULL);
                SDL_Flip(screen);
                break;
            }
            case SDL_KEYDOWN: {
                switch(event.key.keysym.sym) {
                    case SDLK_q:
                    case SDLK_ESCAPE: {
                        run     = 0;
                        break;
                    }
                    case SDLK_r: {
                        record  = !record;
                        printf("\n- recording mode %s\n", record ? "started" : "terminated");
                        break;
                    }
                    case SDLK_f: {
                        fullscreen = fullscreen ? 0 : SDL_FULLSCREEN;
                        if(!doresize) {
                            // do nothing
                        } else if(fullscreen == SDL_FULLSCREEN) {
                            FULLSCREEN_JOB
                        } else {
                            x = xcur;
                            y = ycur;
                        }
                        SDL_FreeSurface(screen);
                        screen = SDL_SetVideoMode(x, y, 0, SDL_ANYFORMAT | SDL_HWSURFACE | SDL_DOUBLEBUF | doresize | fullscreen);
                        if(!screen) sdl_err();
                        SDL_ShowCursor(fullscreen ? SDL_DISABLE : SDL_ENABLE);
                        if(doresize && (x != wold)) image = SDL_ResizeXY(image, x, y, RESIZEQUAL);
                        SDL_BlitSurface(image, NULL, screen, NULL);
                        SDL_Flip(screen);
                        break;
                    }
                    default: break;
                }
                break;
            }
            case SDL_QUIT: {
                run = 0;
                break;
            }
            default: break;
        }
        ready = 1;
    }

    SDL_KillThread(tid);
    SDL_WaitThread(tid, NULL);
    SDL_FreeSurface(image);
    return(0);
}



void show_examples(void) {
    printf("\n"
        "Usage examples:\n"
        "\n"
        "refresh a remote image each second:\n"
        "  webimgms -t 1000 http://example.com/image.jpg\n"
        "  webimgms -t 1000 \"http://example.com/images.php?num=123\"\n"
        "refresh a local image each second:\n"
        "  webimgms -t 1000 c:\\folder\\image.jpg\n"
        "show a collection of images like 00000000.jpg, 00000001.jpg and so on:\n"
        "  webimgms -f \"%%08x.jpg\" c:\\folder\\\n"
        "  webimgms -f \"%%08x.jpg\" c:\\folder\\00000000.jpg\n"
        "as above but with images like 0.jpg, 1.jpg, 2.jpg, 3.jpg and so on:\n"
        "  webimgms -f \"%%d.jpg\" c:\\folder\n"
        "as above but with images like image0000.jpg, image0001.jpg and so on:\n"
        "  webimgms -f \"%%04d.jpg\" c:\\folder\\image\n"
        "  webimgms -f \"image%%04d.jpg\" c:\\folder\\\n"
        "as above but remote and with images like image0.jpg, image1.jpg and so on:\n"
        "  webimgms -f \"%%d.jpg\" http://example.com/folder/image\n"
        "  webimgms -f \"%%d.jpg\" http://example.com/folder/image\n"
        "and now starting from file 1 instead of 0:\n"
        "  webimgms -n 1 -f \"%%d.jpg\" http://example.com/folder/image\n"
        "slide show of the files contained in the folder myimages each 100 ms:\n"
        "  webimgms -t 100 -s myimages\n"
        "\n"
        "for more informations about the FMT format take a look here:\n"
        "  http://en.wikipedia.org/wiki/Printf#printf_format_placeholders\n"
        "\n"
        "Remember that when the -f option is in use the tool checks if the first link\n"
        "already contains the formatted filename of the first file to load (valid for\n"
        "both local and remote files) so the following links will be handled correctly\n"
        "(-f \"%%08x.jpg\" -n 1):\n"
        "  http://example.com/folder/00000001.jpg\n"
        "  http://example.com/folder/\n"
        "while the following are incorrect:\n"
        "  http://example.com/folder                 (will become folder00000001.jpg)\n"
        "  http://example.com/folder/12345678.jpg    (this is not the first file)\n"
        "anyway don't worry since the tool will show ALL the needed details at runtime\n"
        "\n");
    exit(1);
}



int recursive_dir(u8 *filedir) {
    int     plen,
            ret     = -1;

#ifdef WIN32
    static int      winnt = -1;
    OSVERSIONINFO   osver;
    WIN32_FIND_DATA wfd;
    HANDLE  hFind;

    if(winnt < 0) {
        osver.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
        GetVersionEx(&osver);
        if(osver.dwPlatformId >= VER_PLATFORM_WIN32_NT) {
            winnt = 1;
        } else {
            winnt = 0;
        }
    }

    plen = strlen(filedir);
    strcpy(filedir + plen, "\\*.*");
    plen++;

    if(winnt) { // required to avoid problems with Vista and Windows7!
        hFind = FindFirstFileEx(filedir, FindExInfoStandard, &wfd, FindExSearchNameMatch, NULL, 0);
    } else {
        hFind = FindFirstFile(filedir, &wfd);
    }
    if(hFind == INVALID_HANDLE_VALUE) return(0);
    do {
        if(!strcmp(wfd.cFileName, ".") || !strcmp(wfd.cFileName, "..")) continue;

        strcpy(filedir + plen, wfd.cFileName);

        if(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
            if(recursive_dir(filedir) < 0) goto quit;
        } else {
            letsfile(filedir);  // without + 2
        }
    } while(FindNextFile(hFind, &wfd));
    ret = 0;

quit:
    FindClose(hFind);
#else
    struct  stat    xstat;
    struct  dirent  **namelist;
    int     n,
            i;

    n = scandir(filedir, &namelist, NULL, NULL);
    if(n < 0) {
        if(stat(filedir, &xstat) < 0) {
            printf("**** %s", filedir);
            std_err();
        }
        letsfile(filedir);  // without + 2
        return(0);
    }

    plen = strlen(filedir);
    strcpy(filedir + plen, "/");
    plen++;

    for(i = 0; i < n; i++) {
        if(!strcmp(namelist[i]->d_name, ".") || !strcmp(namelist[i]->d_name, "..")) continue;
        strcpy(filedir + plen, namelist[i]->d_name);

        if(stat(filedir, &xstat) < 0) {
            printf("**** %s", filedir);
            std_err();
        }
        if(S_ISDIR(xstat.st_mode)) {
            if(recursive_dir(filedir) < 0) goto quit;
        } else {
            letsfile(filedir);  // without + 2
        }
        free(namelist[i]);
    }
    ret = 0;

quit:
    for(; i < n; i++) free(namelist[i]);
    free(namelist);
#endif
    filedir[plen - 1] = 0;
    return(ret);
}



u8 *get_date(void) {
    time_t  datex;
    struct  tm  *tmx;
    static u8  folder[32];
    static const u8 *months[12] = {
                "jan","feb","mar","apr","may","jun",
                "jul","aug","sep","oct","nov","dec"
            };

    time(&datex);
    tmx = localtime(&datex);
    sprintf(
        folder,
        "%02u.%s.%02u-%02u.%02u",
        tmx->tm_mday, months[tmx->tm_mon], tmx->tm_year % 100,
        tmx->tm_hour, tmx->tm_min);
    return(folder);
}



int letsdownload(void *parameter) {
    mydown_options  opt;
    u32     timez;
    int     sd,
            err     = 0,
            datasz,
            first   = 1;
    u8      *data   = NULL;

    memset(&opt, 0, sizeof(opt));
    sd = 0;
    opt.keep_alive  = &sd;
    opt.filedata    = &data;
    opt.verbose     = 1;
    opt.timeout     = 5;

    while(run) {
        timez = SDL_GetTicks();
        while(!ready) SDL_Delay(5);
        if(fmt) do_fmt();

        if(first) printf("- check the existence of %s\n", url);

        datasz = mydown(url, NULL, &opt);
        if(datasz == MYDOWN_ERROR) {
            if(sd) {
                close(sd);
                sd = 0;
            }
            if(++err < 8) {
                if(doskip) start++;
                continue;
            }
            if(!first && fmt && loop) {
                printf("\n- restart loop\n");
                start = startb;
                continue;
            }
            break;
        }
        if(!datasz) {
            if(sd) {
                close(sd);
                sd = 0;
            }
            continue;
        }
        if(err) err = 0;
        if(first) {
            opt.verbose = -1;
            first = 0;
        } else {
            fputc('.', stdout);
        }
        if(rw.hidden.mem.base) free(rw.hidden.mem.base);
        rw.hidden.mem.base = data;
        rw.hidden.mem.here = data;
        rw.hidden.mem.stop = data + datasz;

        if(record) dumpimg(data, datasz);

        pushrefresh(SHOWIT);
        timez = SDL_GetTicks() - timez;
        if(timez < ms) SDL_Delay(ms - timez);
        start++;
    }
    run = 0;
    if(rw.hidden.mem.base) free(rw.hidden.mem.base);
    rw.hidden.mem.base = NULL;
    return(0);
}



int letsfile(u8 *fname) {
    struct  stat    xstat;
    FILE    *fd;
    u32     timez;
    int     datasz;
    u8      *data;

    while(!ready) SDL_Delay(5);
    timez = SDL_GetTicks();

    fd = fopen(fname, "rb");
    if(!fd) return(-1);
    fstat(fileno(fd), &xstat);
    datasz = xstat.st_size;
    data = malloc(datasz);
    if(!data) std_err();
    fread(data, 1, datasz, fd);
    fclose(fd);
    fputc('.', stdout);

    if(rw.hidden.mem.base) free(rw.hidden.mem.base);
    rw.hidden.mem.base = data;
    rw.hidden.mem.here = data;
    rw.hidden.mem.stop = data + datasz;

    if(record) dumpimg(data, datasz);

    pushrefresh(SHOWIT);
    timez = SDL_GetTicks() - timez;
    if(timez < ms) SDL_Delay(ms - timez);
    start++;
    return(0);
}



int letsload(void *parameter) {
    struct  stat    xstat;

    if(fmt) do_fmt();
    printf("- check the existence of %s\n", url);
    if(stat(url, &xstat) < 0) std_err();

    while(run) {
        if(fmt) do_fmt();

        if(letsfile(url)) {
            if(fmt && loop) {
                start = startb;
                printf("\n- restart loop\n");
                continue;
            }
            fputc('x', stdout);
            break;
        }
    }
    run = 0;
    if(rw.hidden.mem.base) free(rw.hidden.mem.base);
    rw.hidden.mem.base = NULL;
    return(0);
}



int letsslideshow(void *parameter) {
    struct  stat    xstat;
    u8      filedir[PATHMAX + 1];

    printf("- slide show of %s\n", url);
    if(stat(url, &xstat) < 0) std_err();

    while(run) {
        strncpy(filedir, url, sizeof(filedir));
        filedir[sizeof(filedir) - 1] = 0;

        if(recursive_dir(filedir) < 0) {
            if(!loop) break;
        }
        printf("\n- restart loop\n");
    }
    run = 0;
    if(rw.hidden.mem.base) free(rw.hidden.mem.base);
    rw.hidden.mem.base = NULL;
    return(0);
}




void do_fmt(void) {
    int     len;
    static  u8  *add    = NULL;
    u8      tmp[MAXFMT + 1],
            *p;

    if(!fmt) return;
    if(!add) {
        len = snprintf(tmp, sizeof(tmp), fmt, start);
        if((len < 0) || (len >= sizeof(tmp))) goto quit;

        p = strrchr(url, '\\');
        if(!p) p = strrchr(url, '/');
        if(p) {
            p++;
            if(stricmp(p, tmp)) p += strlen(p);
        } else {
            p = url + strlen(url);
        }
        add = p;
    }
    len = snprintf(add, urllen - (add - url), fmt, start);
    if((len < 0) || (len >= urllen)) goto quit;
    return;
quit:
    printf("\nError: your format or your URL is too big\n");
    exit(1);
}



void dumpimg(u8 *data, int datasz) {
    FILE    *fd;
    u32     crc;
    static u32  oldcrc = 0;
    static int  imgnum = 0;
    static u8   fname[MAX_PATH + 32],
                *dumpfolder = NULL,
                *imgname;

    if(!datasz) return;

    if(!dumpfolder) {
        dumpfolder = get_date();
        make_dir(dumpfolder);
        imgname = fname + sprintf(fname, "%s%c", dumpfolder, PATH_SLASH);
        printf("\n- images are recorded in the folder %s using the format %%09u.jpg\n", dumpfolder);
    }

    crc = mycrc(data, datasz);
    if(crc == oldcrc) return;
    oldcrc = crc;

    sprintf(imgname, "%09u.jpg", imgnum);   // 10 is better but 9 has a better visualization
    fd = fopen(fname, "wb");
    if(!fd) return;
    fwrite(data, 1, datasz, fd);
    fclose(fd);
    imgnum++;
}



void pushrefresh(int code) {
    SDL_Event   event;

    event.type       = SDL_USEREVENT;
    event.user.code  = code;
    event.user.data1 = NULL;
    event.user.data2 = NULL;
    ready = 0;
    SDL_PushEvent(&event);
}



void pushevent(int type, int code) {
    SDL_Event   event;

    event.type       = type;
    event.user.code  = code;
    event.user.data1 = NULL;
    event.user.data2 = NULL;
    ready = 0;
    SDL_PushEvent(&event);
}



void pushfullscreen(void) {
    SDL_Event   event;

    event.type       = SDL_KEYDOWN;
    event.user.code  = 0;
    event.user.data1 = NULL;
    event.user.data2 = NULL;
    event.key.keysym.sym = SDLK_f;
    ready = 0;
    SDL_PushEvent(&event);
}



void write_watch_ever(void) {
    FILE    *fd;

    fd = fopen("rewatch.htm", "wb");
    if(!fd) std_err();

    fprintf(fd,
        "<html>\n"
        "<title>webimgms</title>\n"
        "<head>\n"
        "<style type=\"text/css\"><!--*{margin:0;padding:0;}--></style>\n"
        "<script language=\"JavaScript\">\n"
        "<!--\n"
        "    var i = 0;\n"
        "    setTimeout(\"loadimg()\", 1);\n"
        "\n"
        "    function d2h(d) {\n"
        "        var hex = \"0123456789abcdef\";\n"
        "        var h = \"\";\n"
        "        var j;\n"
        "        for(j = 0; j < 8; j++) {\n"
        "            h = hex.substr(d & 15, 1) + h;\n"
        "            d >>= 4;\n"
        "        }\n"
        "        return h;\n"
        "    }\n"
        "\n"
        "    function loadimg() {\n"
        "        document.myimg.src = d2h(i) + \".jpg\";\n"
        "        i++;\n"
        "        setTimeout(\"loadimg()\", %d);\n"
        "    }\n"
        "-->\n"
        "</script>\n"
        "</head><body>\n"
        "<img name=\"myimg\" src=\"00000000.jpg\" width=100%% border=0>\n"
        "</body></html>\n", ms);

    fclose(fd);
}



u32 get_num(u8 *str) {
    u32     num;

    if((str[0] == '0') && (tolower(str[1]) == 'x')) {
        sscanf(str + 2, "%x", &num);
    } else {
        sscanf(str, "%u", &num);
    }
    return(num);
}



u32 mycrc(u8 *data, int size) {
    const static u32   crctable[] = {
        0x00000000, 0x77073096, 0xee0e612c, 0x990951ba,
        0x076dc419, 0x706af48f, 0xe963a535, 0x9e6495a3,
        0x0edb8832, 0x79dcb8a4, 0xe0d5e91e, 0x97d2d988,
        0x09b64c2b, 0x7eb17cbd, 0xe7b82d07, 0x90bf1d91,
        0x1db71064, 0x6ab020f2, 0xf3b97148, 0x84be41de,
        0x1adad47d, 0x6ddde4eb, 0xf4d4b551, 0x83d385c7,
        0x136c9856, 0x646ba8c0, 0xfd62f97a, 0x8a65c9ec,
        0x14015c4f, 0x63066cd9, 0xfa0f3d63, 0x8d080df5,
        0x3b6e20c8, 0x4c69105e, 0xd56041e4, 0xa2677172,
        0x3c03e4d1, 0x4b04d447, 0xd20d85fd, 0xa50ab56b,
        0x35b5a8fa, 0x42b2986c, 0xdbbbc9d6, 0xacbcf940,
        0x32d86ce3, 0x45df5c75, 0xdcd60dcf, 0xabd13d59,
        0x26d930ac, 0x51de003a, 0xc8d75180, 0xbfd06116,
        0x21b4f4b5, 0x56b3c423, 0xcfba9599, 0xb8bda50f,
        0x2802b89e, 0x5f058808, 0xc60cd9b2, 0xb10be924,
        0x2f6f7c87, 0x58684c11, 0xc1611dab, 0xb6662d3d,
        0x76dc4190, 0x01db7106, 0x98d220bc, 0xefd5102a,
        0x71b18589, 0x06b6b51f, 0x9fbfe4a5, 0xe8b8d433,
        0x7807c9a2, 0x0f00f934, 0x9609a88e, 0xe10e9818,
        0x7f6a0dbb, 0x086d3d2d, 0x91646c97, 0xe6635c01,
        0x6b6b51f4, 0x1c6c6162, 0x856530d8, 0xf262004e,
        0x6c0695ed, 0x1b01a57b, 0x8208f4c1, 0xf50fc457,
        0x65b0d9c6, 0x12b7e950, 0x8bbeb8ea, 0xfcb9887c,
        0x62dd1ddf, 0x15da2d49, 0x8cd37cf3, 0xfbd44c65,
        0x4db26158, 0x3ab551ce, 0xa3bc0074, 0xd4bb30e2,
        0x4adfa541, 0x3dd895d7, 0xa4d1c46d, 0xd3d6f4fb,
        0x4369e96a, 0x346ed9fc, 0xad678846, 0xda60b8d0,
        0x44042d73, 0x33031de5, 0xaa0a4c5f, 0xdd0d7cc9,
        0x5005713c, 0x270241aa, 0xbe0b1010, 0xc90c2086,
        0x5768b525, 0x206f85b3, 0xb966d409, 0xce61e49f,
        0x5edef90e, 0x29d9c998, 0xb0d09822, 0xc7d7a8b4,
        0x59b33d17, 0x2eb40d81, 0xb7bd5c3b, 0xc0ba6cad,
        0xedb88320, 0x9abfb3b6, 0x03b6e20c, 0x74b1d29a,
        0xead54739, 0x9dd277af, 0x04db2615, 0x73dc1683,
        0xe3630b12, 0x94643b84, 0x0d6d6a3e, 0x7a6a5aa8,
        0xe40ecf0b, 0x9309ff9d, 0x0a00ae27, 0x7d079eb1,
        0xf00f9344, 0x8708a3d2, 0x1e01f268, 0x6906c2fe,
        0xf762575d, 0x806567cb, 0x196c3671, 0x6e6b06e7,
        0xfed41b76, 0x89d32be0, 0x10da7a5a, 0x67dd4acc,
        0xf9b9df6f, 0x8ebeeff9, 0x17b7be43, 0x60b08ed5,
        0xd6d6a3e8, 0xa1d1937e, 0x38d8c2c4, 0x4fdff252,
        0xd1bb67f1, 0xa6bc5767, 0x3fb506dd, 0x48b2364b,
        0xd80d2bda, 0xaf0a1b4c, 0x36034af6, 0x41047a60,
        0xdf60efc3, 0xa867df55, 0x316e8eef, 0x4669be79,
        0xcb61b38c, 0xbc66831a, 0x256fd2a0, 0x5268e236,
        0xcc0c7795, 0xbb0b4703, 0x220216b9, 0x5505262f,
        0xc5ba3bbe, 0xb2bd0b28, 0x2bb45a92, 0x5cb36a04,
        0xc2d7ffa7, 0xb5d0cf31, 0x2cd99e8b, 0x5bdeae1d,
        0x9b64c2b0, 0xec63f226, 0x756aa39c, 0x026d930a,
        0x9c0906a9, 0xeb0e363f, 0x72076785, 0x05005713,
        0x95bf4a82, 0xe2b87a14, 0x7bb12bae, 0x0cb61b38,
        0x92d28e9b, 0xe5d5be0d, 0x7cdcefb7, 0x0bdbdf21,
        0x86d3d2d4, 0xf1d4e242, 0x68ddb3f8, 0x1fda836e,
        0x81be16cd, 0xf6b9265b, 0x6fb077e1, 0x18b74777,
        0x88085ae6, 0xff0f6a70, 0x66063bca, 0x11010b5c,
        0x8f659eff, 0xf862ae69, 0x616bffd3, 0x166ccf45,
        0xa00ae278, 0xd70dd2ee, 0x4e048354, 0x3903b3c2,
        0xa7672661, 0xd06016f7, 0x4969474d, 0x3e6e77db,
        0xaed16a4a, 0xd9d65adc, 0x40df0b66, 0x37d83bf0,
        0xa9bcae53, 0xdebb9ec5, 0x47b2cf7f, 0x30b5ffe9,
        0xbdbdf21c, 0xcabac28a, 0x53b39330, 0x24b4a3a6,
        0xbad03605, 0xcdd70693, 0x54de5729, 0x23d967bf,
        0xb3667a2e, 0xc4614ab8, 0x5d681b02, 0x2a6f2b94,
        0xb40bbe37, 0xc30c8ea1, 0x5a05df1b, 0x2d02ef8d };
    u32    crc = 0xffffffff;

    while(size--) {
        crc = crctable[*data ^ (crc & 0xff)] ^ (crc >> 8);
        data++;
    }
    return(~crc);
}



void sdl_err(void) {
    printf("\nError: %s\n", SDL_GetError());
    exit(1);
}



void std_err(void) {
    perror("\nError");
    exit(1);
}



    // from SDL_rwops.c
int my_mem_seek(SDL_RWops *context, int offset, int whence)
{
	Uint8 *newpos;

	switch (whence) {
		case RW_SEEK_SET:
			newpos = context->hidden.mem.base+offset;
			break;
		case RW_SEEK_CUR:
			newpos = context->hidden.mem.here+offset;
			break;
		case RW_SEEK_END:
			newpos = context->hidden.mem.stop+offset;
			break;
		default:
			SDL_SetError("Unknown value for 'whence'");
			return(-1);
	}
	if ( newpos < context->hidden.mem.base ) {
		newpos = context->hidden.mem.base;
	}
	if ( newpos > context->hidden.mem.stop ) {
		newpos = context->hidden.mem.stop;
	}
	context->hidden.mem.here = newpos;
	return(context->hidden.mem.here-context->hidden.mem.base);
}
int my_mem_read(SDL_RWops *context, void *ptr, int size, int maxnum)
{
	size_t total_bytes;
	size_t my_mem_available;

	total_bytes = (maxnum * size);
	if ( (maxnum <= 0) || (size <= 0) || ((total_bytes / maxnum) != (size_t) size) ) {
		return 0;
	}

	my_mem_available = (context->hidden.mem.stop - context->hidden.mem.here);
	if (total_bytes > my_mem_available) {
		total_bytes = my_mem_available;
	}

	memcpy(ptr, context->hidden.mem.here, total_bytes);
	context->hidden.mem.here += total_bytes;

	return (total_bytes / size);
}
int my_mem_write(SDL_RWops *context, const void *ptr, int size, int num)
{
	if ( (context->hidden.mem.here + (num*size)) > context->hidden.mem.stop ) {
		num = (context->hidden.mem.stop-context->hidden.mem.here)/size;
	}
	memcpy(context->hidden.mem.here, ptr, num*size);
	context->hidden.mem.here += num*size;
	return(num);
}
int my_mem_close(SDL_RWops *context)
{
	if ( context ) {
		SDL_FreeRW(context);
	}
	return(0);
}


